
#include "crypto_hashblocks_sha256.h"

#define crypto_hashblocks crypto_hashblocks_sha256
#define crypto_hashblocks_STATEBYTES crypto_hashblocks_sha256_STATEBYTES
#define crypto_hashblocks_BLOCKBYTES crypto_hashblocks_sha256_BLOCKBYTES
#define crypto_hashblocks_PRIMITIVE "sha256"
#define crypto_hashblocks_IMPLEMENTATION crypto_hashblocks_sha256_IMPLEMENTATION
#define crypto_hashblocks_VERSION crypto_hashblocks_sha256_VERSION
